import json
import os
import re
import warnings


def try_parse_error(e):
    try:
        json_str = e.read()
        return json.loads(json_str)
    except KeyboardInterrupt as e:
        raise e
    except SystemExit as e:
        raise e
    except Exception as e:
        warnings.warn(str(e))
        return None


def readfile_or_default(fp, default=''):
    if os.path.exists(fp):
        with open(fp) as fd:
            return fd.read()
    return default


def count_slashes(input_string):
    dot_count = 0
    for char in input_string:
        if char == '/':
            dot_count += 1
    return dot_count


def is_name_legit(input_string):
    return bool(re.match(r'^(?=[a-zA-Z])[a-zA-Z0-9_-]*$', input_string))


def parse_dataset_name(name: str):
    dot_count = count_slashes(name)
    if dot_count == 0:
        store_name, dataset_name = 'main', name
    elif dot_count == 1:
        store_name, dataset_name = name.split('/')
    else:
        raise ValueError('dataset name must be [store_name]/[dataset_name] or [dataset_name]')

    if not is_name_legit(store_name) or not is_name_legit(dataset_name):
        raise ValueError('dataset name and store name must startswith a-zA-Z, and contains only [a-zA-Z0-9_-]')

    return store_name, dataset_name


def read_jsonl(line_stream):
    for line in line_stream:
        try:
            yield json.loads(line)
        except json.decoder.JSONDecodeError:
            pass



import datetime
import os
from dataclasses import dataclass
from typing import List, Optional

from pydantic import BaseModel, Field


@dataclass
class DatasetInfo:
    name: str
    is_redirect: bool


class DatasetInfoSnippet(BaseModel):
    datastore: str
    dataset: str
    readme: str


class StoreList(BaseModel):
    stores: List[str]


class DatasetListingResults(BaseModel):
    items: List[DatasetInfoSnippet]


class SourceInfo(BaseModel):
    url: str
    download_time: datetime.datetime = Field(default_factory=datetime.datetime.now)


class DatasetHomeFolderFilenames:
    MARKER = '.dataset.marker'
    META = 'meta.json'
    DATA_FILE = 'file'
    README = 'readme.md'
    REMOTE_SOURCE = 'remote_source.json'  # if this file is downloaded from a remote source,


class DatasetFileInternalPath:
    BINARY_FOLDER = 'bin'
    COLLECTION_FOLDER = 'coll'

    META_FILE_NAME = 'meta.json'
    DATA_FILE = 'data.jsonl'


@dataclass
class DatasetHomeFolder:
    base: str
    name: Optional[str] = None

    def meta(self):
        return os.path.join(self.base, DatasetHomeFolderFilenames.META)

    def marker(self):
        return os.path.join(self.base, DatasetHomeFolderFilenames.MARKER)

    def datafile(self):
        return os.path.join(self.base, DatasetHomeFolderFilenames.DATA_FILE)

    def readme(self):
        return os.path.join(self.base, DatasetHomeFolderFilenames.README)

    def marker_exist(self):
        return os.path.isfile(self.marker())

    def remote_source(self):
        return os.path.join(self.base, DatasetHomeFolderFilenames.REMOTE_SOURCE)

    def code_example(self, collection_name):
        return os.path.join(self.base, f'usage_code_{collection_name}.py')

    def sample_file(self, collection_name):
        return os.path.join(self.base, f'data_sample_{collection_name}.jsonl')



import json
import os
import shutil
import tempfile
import time
import urllib
import urllib.request
import uuid
import warnings
import zipfile
from typing import Optional, Type, TypeVar, List
from urllib.error import HTTPError

from pydantic import BaseModel

from .config import get_storage_base_folder
from .models import DatasetInfoSnippet, StoreList, DatasetListingResults, DatasetHomeFolder, \
    SourceInfo, DatasetFileInternalPath
from .typing.codegen import CodeGenerator
from .typing.schema_builder import SchemaBuilder
from .utils.misc import readfile_or_default, is_name_legit, parse_dataset_name, read_jsonl, try_parse_error
from .core import CollectionConfig, DatasetFileMeta

DataModel = TypeVar('DataModel', bound=BaseModel)


class DatasetFile:

    def __init__(self):
        raise ValueError('Please use DatasetFile.open(filename, mode)')

    @staticmethod
    def open(fp: str, mode: str = 'r'):
        """
        Open a dataset file
        :param fp: path to the file
        :param mode: r for read and w for write.
        :return:
        """
        if mode == 'r':
            return DatasetFileReader(fp)
        elif mode == 'w':
            return DatasetFileWriter(fp)
        else:
            raise ValueError('mode must be one of "r" or "w"')

    @staticmethod
    def binary_file_path(fn: str):
        return os.path.join(DatasetFileInternalPath.BINARY_FOLDER, fn)


class DatasetFileWriter:
    def __init__(self, file_path: str):
        """
        Write to a dataset file, this object can also be used as a context manager.

        This object need to be closed.

        :param file_path: location of the dataset file to write.
        """
        self.zip_file = zipfile.ZipFile(file_path, 'w')
        self.meta = DatasetFileMeta()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()

    def close(self):
        """
        close the writer.
        :return:
        """
        self.meta.createdAt = int(time.time())
        with self.zip_file.open(DatasetFileInternalPath.META_FILE_NAME, 'w') as out:
            out.write(self.meta.model_dump_json().encode('utf-8'))
        self.zip_file.close()

    def add_collection(
            self,
            collection_name: str,
            data: List,
            model: Optional[Type[DataModel]] = None,
    ):
        """
        add a data collection to this dataset.
        :param collection_name: name of the collection to add.
        :param data: list of objects that extends pydantic BaseModel.
        :param model: the pydantic model class.
        :return:
        """
        for coll in self.meta.collections:
            if coll.name == collection_name:
                raise ValueError(f'collection {collection_name} already exists')

        if model is None:
            if isinstance(data[0], BaseModel):
                warnings.warn(f'model class is not provided, using {data[0].__class__.__name__} as model')
                model = data[0].__class__
            else:
                raise ValueError('Input data must be pydantic model')

        dataset_schema = SchemaBuilder.build(model)

        new_coll = CollectionConfig(
            name=collection_name,
            data_schema=dataset_schema,
        )

        self.meta.collections.append(new_coll)
        target_fp = os.path.join(
            DatasetFileInternalPath.COLLECTION_FOLDER,
            collection_name,
            DatasetFileInternalPath.DATA_FILE
        )
        with self.zip_file.open(target_fp, 'w') as out:
            for item in data:
                out.write(item.model_dump_json(round_trip=True).encode('utf-8'))
                out.write("\n".encode('utf-8'))

    def add_binary_file(self, fn: str, content: bytes):
        """
        Add a binary file to the dataset
        :param fn: name of the binary file.
        :param content: content in bytes.
        :return:
        """
        binary_file_path = DatasetFile.binary_file_path(fn)
        with self.zip_file.open(binary_file_path, 'w') as out:
            out.write(content)


class DatasetFileReader:
    def __init__(self, file_path):
        """
        Read a dataset, this object can be used as a context manager.

        This object must be closed.

        :param file_path:
        """
        self.zip_file = zipfile.ZipFile(file_path, 'r')

        with self.zip_file.open(DatasetFileInternalPath.META_FILE_NAME, 'r') as fd:
            self.meta = DatasetFileMeta(**json.load(fd))

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()

    def close(self):
        self.zip_file.close()

    def binary_files(self):
        """
        Open a binary file for read.
        :return: a file descriptor for the binary file to read.
        """
        prefix = DatasetFileInternalPath.BINARY_FOLDER + '/'
        for name in self.zip_file.namelist():
            if name.startswith(prefix):
                yield name[len(prefix):]

    def open_binary_file(self, filename):
        """
        Open a binary file for read.
        :param filename: name of the binary file.
        :return: a file descriptor for the binary file to read.
        """
        return self.zip_file.open(
            DatasetFile.binary_file_path(filename),
            'r'
        )

    def collection(self, collection_name, model=None):
        """
        Open a collection.
        :param collection_name: name of a collection
        :param model: an optional pydantic model class to hold the data.
        :return: a CollectionReader object for the given collection name.
        """
        cfg = [c for c in self.meta.collections if c.name == collection_name]
        if len(cfg) == 0:
            raise ValueError(f"Collection {collection_name} do not exist")
        else:
            cfg = cfg[0]
        return CollectionReader(self.zip_file, collection_name, cfg, model=model)

    def coll(self, collection_name, model=None):
        return self.collection(collection_name, model=model)

    def collections(self):
        """
        List all collection names
        :return: list of collection names.
        """
        return [c.name for c in self.meta.collections]

    def __getitem__(self, item):
        return self.collection(item)


class CollectionReader(object):
    def __init__(self, zip_file, collection_name, config: CollectionConfig, model=None):
        """
        Collection Reader
        :param zip_file:
        :param collection_name:
        :param config:
        :param model:
        """
        self.zip_file = zip_file
        self.collection_name = collection_name
        self.config = config
        self.model = model

    def code_usage(self):
        generator = CodeGenerator()
        code = generator.generate_all(self.config.data_schema)
        return code

    def samples(self, n=10):
        ret = []
        for i, row in enumerate(self):
            if i >= n:
                break
            ret.append(row)
        return ret

    def __iter__(self):
        """
        Iterate through the collection.
        :return:
        """
        entry = os.path.join(
            DatasetFileInternalPath.COLLECTION_FOLDER,
            self.collection_name,
            DatasetFileInternalPath.DATA_FILE
        )
        with self.zip_file.open(entry, 'r') as fd:
            for line in fd:
                line = line.strip()
                if len(line) > 0:
                    item = json.loads(line)
                    if self.model is None:
                        yield item
                    else:
                        yield self.model(**item)

    def to_list(self):
        """
        Read the collection as list instead of iterator
        :return:
        """
        return list(self)


class DatasetStorageManager:
    def __init__(self, store_base=None):
        if store_base is None or store_base == '':
            store_base = get_storage_base_folder()
        self.base = store_base

    def list_dataset_stores(self) -> StoreList:
        base_dir = self.base
        stores = []

        if os.path.exists(base_dir):

            for store_name in os.listdir(base_dir):
                store_dir = os.path.join(base_dir, store_name)

                if os.path.isdir(store_dir) and is_name_legit(store_name):
                    stores.append(store_name)

        return StoreList(stores=stores)

    def list_datasets(self) -> DatasetListingResults:
        base_dir = self.base
        datasets = []

        if os.path.exists(base_dir):
            for store_name in os.listdir(base_dir):
                store_dir = os.path.join(base_dir, store_name)

                if os.path.isdir(store_dir):
                    for dataset_name in os.listdir(store_dir):
                        dataset_dir = os.path.join(store_dir, dataset_name)
                        dataset_home_folder = DatasetHomeFolder(dataset_dir)
                        if dataset_home_folder.marker_exist():
                            readme = readfile_or_default(dataset_home_folder.readme())
                            datasets.append(DatasetInfoSnippet(
                                datastore=store_name,
                                dataset=dataset_name,
                                readme=readme
                            ))
        return DatasetListingResults(items=datasets)

    def list_datasets_in_store(self, store_name) -> DatasetListingResults:
        base_dir = self.base
        datasets = []

        store_dir = os.path.join(base_dir, store_name)
        if os.path.isdir(store_dir):
            for dataset_name in os.listdir(store_dir):
                dataset_dir = os.path.join(store_dir, dataset_name)
                dataset_home_folder = DatasetHomeFolder(dataset_dir)
                if dataset_home_folder.marker_exist():
                    readme = readfile_or_default(dataset_home_folder.readme())
                    datasets.append(DatasetInfoSnippet(
                        datastore=store_name,
                        dataset=dataset_name,
                        readme=readme
                    ))
        return DatasetListingResults(items=datasets)

    def get_dataset_dir(self, store_name, dataset_name):
        store_dir = os.path.join(self.base, store_name, dataset_name)
        return DatasetHomeFolder(store_dir)

    def get_dataset_meta(self, store_name, dataset_name):
        dataset_home_folder = self.get_dataset_dir(store_name, dataset_name)
        return json.loads(readfile_or_default(dataset_home_folder.meta()))

    def get_dataset_source_info(self, store_name, dataset_name) -> Optional[SourceInfo]:
        dataset_home_folder = self.get_dataset_dir(store_name, dataset_name)
        source_json = readfile_or_default(dataset_home_folder.remote_source(), None)
        if source_json is not None:
            return SourceInfo(**json.loads(source_json))
        return None

    def get_dataset_file_path(self, store_name, dataset_name):
        dataset_home_folder = self.get_dataset_dir(store_name, dataset_name)
        return dataset_home_folder.datafile()

    def get_dataset_readme(self, store_name, dataset_name):
        dataset_home_folder = self.get_dataset_dir(store_name, dataset_name)
        return readfile_or_default(dataset_home_folder.readme())

    def get_sample(self, store_name, dataset_name, collection):
        dataset_home_folder = self.get_dataset_dir(store_name, dataset_name)
        if os.path.isfile(dataset_home_folder.sample_file(collection)):
            with open(dataset_home_folder.sample_file(collection)) as fd:
                return list(read_jsonl(fd))
        return []

    def get_usage_code(self, store_name, dataset_name, collection_name):
        dataset_home_folder = self.get_dataset_dir(store_name, dataset_name)
        return readfile_or_default(dataset_home_folder.code_example(collection_name))

    def dataset_exist(self, store_name, dataset_name):
        dataset_home_folder = self.get_dataset_dir(store_name, dataset_name)
        return dataset_home_folder.marker_exist()

    def delete_dataset(self, store_name, dataset_name):
        dataset_home_folder = self.get_dataset_dir(store_name, dataset_name)
        shutil.rmtree(dataset_home_folder.base)

    def import_file(self, file, store_name, dataset_name, source_url=None):

        if self.dataset_exist(store_name, dataset_name):
            raise ValueError(f'dataset already exists under the name {store_name}/{dataset_name}')

        dataset_folder = self.get_dataset_dir(store_name, dataset_name)
        os.makedirs(
            dataset_folder.base, exist_ok=True
        )
        shutil.copy2(file, dataset_folder.datafile())
        self.extract_sample_and_code_usage(dataset_folder.datafile(), dataset_folder)

        if source_url is not None:
            with open(dataset_folder.remote_source(), 'w') as out:
                info = SourceInfo(url=source_url)
                json.dump(info.model_dump(mode='json'), out)

        with open(dataset_folder.marker(), 'w') as out:
            out.write('1')

    def import_url(self, url, store_name, dataset_name):
        with tempfile.TemporaryDirectory() as temp_dir:
            file_name = str(uuid.uuid4())
            temp_file_path = os.path.join(temp_dir, file_name)
            try:
                urllib.request.urlretrieve(url, temp_file_path)
            except HTTPError as e:
                server_err = try_parse_error(e)
                if server_err is not None:
                    if 'reason' in server_err:
                        warnings.warn(server_err.get('reason', ''))
                raise e
            self.import_file(temp_file_path, store_name, dataset_name, source_url=url)
        return self.get_dataset_file_path(store_name, dataset_name)

    @staticmethod
    def extract_sample_and_code_usage(data_file, dataset_folder):
        with read_file(data_file) as reader:
            meta_file_dest = dataset_folder.meta()
            with reader.zip_file.open(DatasetFileInternalPath.META_FILE_NAME, 'r') as fd:
                with open(meta_file_dest, 'wb') as out:
                    out.write(fd.read())

            for coll_name in reader.collections():
                coll = reader.collection(coll_name)
                with open(dataset_folder.sample_file(coll_name), 'w') as out:
                    samples = coll.samples(n=10)
                    for item in samples:
                        out.write(f'{json.dumps(item)}\n')
                with open(dataset_folder.code_example(coll_name), 'w') as out:
                    code = coll.code_usage()
                    out.write(code)


# Standard IO operations

def create(fp: str) -> 'DatasetFileWriter':
    """
    Create a dataset file to write
    :param fp: path to the file
    :return:
    """
    return DatasetFileWriter(fp)


def read_file(fp: str) -> 'DatasetFileReader':
    """
    Read a dataset file
    :param fp: path to the file
    :return:
    """
    return DatasetFileReader(fp)


def read(name: str, store_base: Optional[str] = None) -> 'DatasetFileReader':
    """
    Read a managed dataset file by name.
    :param name: name of the dataset file to locate
    :param store_base: where is the base folder or app
    :return:
    """
    manager = DatasetStorageManager(store_base)
    store_name, dataset_name = parse_dataset_name(name)
    located = manager.get_dataset_file_path(store_name, dataset_name)
    if os.path.exists(located):
        return DatasetFileReader(located)
    else:
        raise FileNotFoundError()


def import_file(name, file, store_base=None, exist_ok=True):
    manager = DatasetStorageManager(store_base=store_base)
    store, dataset = parse_dataset_name(name)
    if manager.dataset_exist(store, dataset) and exist_ok:
        print(f'dataset {name} already exists')
        return
    manager.import_file(file, store, dataset)


def import_url(name, url, store_base=None, exist_ok=True):
    manager = DatasetStorageManager(store_base=store_base)
    store, dataset = parse_dataset_name(name)
    if manager.dataset_exist(store, dataset) and exist_ok:
        print(f'dataset {name} already exists')
        return
    manager.import_url(url, store, dataset)


def exists(name, store_base=None):
    manager = DatasetStorageManager(store_base=store_base)
    store, dataset = parse_dataset_name(name)
    return manager.dataset_exist(store, dataset)


def delete_dataset(name, store_base=None):
    manager = DatasetStorageManager(store_base=store_base)
    store, dataset = parse_dataset_name(name)
    return manager.delete_dataset(store, dataset)



from dataset_sh import create


def dump_single_collection(fn, name, data):
    with create(fn) as out:
        out.add_collection(name, data, data[0].__class__)


def dump_collections(fn, data_dict):
    with create(fn) as out:
        for name, data in data_dict.items():
            out.add_collection(name, data, data[0].__class__)



import mimetypes
import os

from flask import Flask, jsonify, send_file, Response
import importlib.resources
import zipfile

from dataset_sh.io import DatasetStorageManager


def load_frontend_assets(prefix='build/'):
    frontend_assets = {}
    with importlib.resources.path("dataset_sh.assets", 'app-ui.frontend') as asset_zip_path:
        if os.path.exists(asset_zip_path):
            with zipfile.ZipFile(asset_zip_path, 'r') as zip_ref:
                for file_info in zip_ref.infolist():
                    if file_info.is_dir():
                        continue
                    if file_info.filename.startswith(prefix):
                        with zip_ref.open(file_info) as file:
                            file_content = file.read()
                            frontend_assets[file_info.filename[len(prefix):]] = file_content
    return frontend_assets


def create_app(manager=None, frontend_assets=None):
    if manager is None:
        manager = DatasetStorageManager()
    app = Flask(__name__, static_folder=None)

    @app.route('/api/dataset', methods=['GET'])
    def list_datasets():
        items = manager.list_datasets()
        return jsonify(items.model_dump(mode='json')), 200

    @app.route('/api/store', methods=['GET'])
    def list_stores():
        items = manager.list_dataset_stores()
        return jsonify(items.model_dump(mode='json')), 200

    @app.route('/api/dataset/<store_name>', methods=['GET'])
    def list_datasets_in_store(store_name):
        items = manager.list_datasets_in_store(store_name)
        return jsonify(items.model_dump(mode='json')), 200

    @app.route('/api/dataset/<store_name>/<dataset_name>/readme', methods=['GET'])
    def get_dataset_readme(store_name, dataset_name):
        return manager.get_dataset_readme(store_name, dataset_name), 200

    @app.route('/api/dataset/<store_name>/<dataset_name>/remote-source', methods=['GET'])
    def get_dataset_remote_source(store_name, dataset_name):
        source = manager.get_dataset_source_info(store_name, dataset_name)
        return jsonify(source.model_dump(mode='json')), 200

    @app.route('/api/dataset/<store_name>/<dataset_name>/meta', methods=['GET'])
    def get_dataset_meta(store_name, dataset_name):
        return jsonify(manager.get_dataset_meta(store_name, dataset_name)), 200

    @app.route('/api/dataset/<store_name>/<dataset_name>/collection/<collection_name>/sample', methods=['GET'])
    def get_collection_sample(store_name, dataset_name, collection_name):
        sample = manager.get_sample(store_name, dataset_name, collection_name)
        return jsonify(sample), 200

    @app.route('/api/dataset/<store_name>/<dataset_name>/collection/<collection_name>/code', methods=['GET'])
    def get_collection_code(store_name, dataset_name, collection_name):
        code = manager.get_usage_code(store_name, dataset_name, collection_name)
        return {'code': code}, 200

    @app.route('/api/dataset/<store_name>/<dataset_name>/file', methods=['GET'])
    def get_dataset_file(store_name, dataset_name):
        return send_file(
            manager.get_dataset_file_path(store_name, dataset_name),
            as_attachment=True,
            download_name=f"{store_name}_{dataset_name}.dataset"
        )

    index_html_file = 'index.html'

    @app.route('/', defaults={'path': ''})
    @app.route("/<string:path>")
    @app.route('/<path:path>')
    def catch_all(path):
        fp = path
        if fp not in frontend_assets:
            # due to frontend routing, we need to serve content index.html of non static asset url.
            if index_html_file not in frontend_assets:
                return '', 404
            fp = index_html_file

        mime_type, _ = mimetypes.guess_type(fp)
        content_bytes = frontend_assets[fp]
        if mime_type is None:
            mime_type = 'application/octet-stream'
        return Response(content_bytes, content_type=mime_type)

    return app


DISABLE_UI = os.environ.get('DISABLE_DATASET_APP_UI', '0') != '0'

_frontend_assets = {
    'build/index.html': "dataset.sh web ui is disabled"
}

if not DISABLE_UI:
    _frontend_assets = load_frontend_assets()

app = create_app(frontend_assets=_frontend_assets)



import os
import json
import warnings

_CONFIG_FILE = '~/.dataset_sh_config.json'


def get_storage_base_folder():
    """
    Read storage base folder path with the following order:
    1. ENV: DATASET_APP_STORAGE_LOCATION
    2. Config File: '~/.dataset_sh_config.json'
    3. DEFAULT_VALUE: '~/dataset_sh/storage'
    :return: storage base folder location
    """
    v = os.environ.get('DATASET_APP_STORAGE_LOCATION', None)
    if v is None:
        cfg_path = os.path.expanduser(_CONFIG_FILE)
        if os.path.exists(cfg_path):
            with open(cfg_path) as fd:
                try:
                    cfg = json.load(fd)
                    if 'storage' in cfg:
                        if 'location' in cfg['storage']:
                            v = cfg['storage']['location']
                except json.decoder.JSONDecodeError:
                    warnings.warn('Cannot parse dataset local config file.')

    if v is None:
        v = '~/dataset_sh/storage'

    return os.path.expanduser(v)



#!/usr/bin/env python3

import json
from dataclasses import dataclass
from typing import Optional

import click

from dataset_sh.core import DatasetFileMeta
from dataset_sh.io import DatasetStorageManager, read_file
from dataset_sh.typing.codegen import CodeGenerator
from dataset_sh.utils.misc import parse_dataset_name


@dataclass
class AppCtx:
    base: Optional[str] = None


@click.group(name='dataset.sh')
@click.option('--base', '-b', envvar='DATASET_SH_STORAGE_BASE', help='location of base storage folder.',
              type=click.Path())
@click.pass_context
def cli(ctx, base):
    """Simple CLI tool with subcommands"""
    ctx.obj = DatasetStorageManager(store_base=base)


@click.command(name='print')
@click.argument('name')
@click.argument('action', type=click.Choice(['list-collections', 'readme', 'code', 'sample']))
@click.option('--collection', '-c', help='which collection')
@click.pass_obj
def print_info(manager: DatasetStorageManager, name, action, collection):
    """
    print dataset information
    """
    store_name, dataset_name = parse_dataset_name(name)

    if action in [
        'code',
        'sample',
    ] and (collection is None or collection == ''):
        click.echo('You must provide a collection using --collection/-c')
        raise click.Abort()

    if action == 'list-collections':
        meta = manager.get_dataset_meta(store_name, dataset_name)
        meta = DatasetFileMeta(**meta)

        click.echo(f'Total collections: {len(meta.collections)}')
        for coll in meta.collections:
            click.echo(coll.name)

    elif action == 'readme':
        readme = manager.get_dataset_readme(store_name, dataset_name)
        click.echo(readme)

    elif action == 'code':
        code = manager.get_usage_code(store_name, dataset_name, collection_name=collection)
        click.echo(code)

        meta = manager.get_dataset_meta(store_name, dataset_name)
        meta = DatasetFileMeta(**meta)
        coll = meta.find_collection(collection)

        if coll is not None:
            loader_code = CodeGenerator.generate_loader_code(
                store_name,
                dataset_name,
                collection,
                coll.data_schema.entry_point)
            click.echo('\n\n')
            click.echo(loader_code)

    elif action == 'sample':
        sample = manager.get_sample(store_name, dataset_name, collection)
        click.echo(json.dumps(sample, indent=2))


@click.command(name='inspect-file')
@click.argument('filepath', type=click.Path())
@click.argument('action', type=click.Choice(['list-collections', 'code', 'sample']))
@click.option('--collection', '-c', help='which collection.')
@click.pass_obj
def inspect_file(manager: DatasetStorageManager, filepath, action, collection):
    """
    parse dataset file and print out information
    """

    if action in [
        'code',
        'sample',
    ] and (collection is None or collection == ''):
        click.echo('You must provide a collection using --collection/-c')
        raise click.Abort()

    with read_file(filepath) as reader:
        if action == 'list-collections':
            collections = reader.collections()
            click.echo(f'Total collections: {len(collections)}')
            for coll in collections:
                click.echo(coll)

        elif action == 'code':
            coll = reader.collection(collection)
            code = coll.code_usage()
            schema = coll.config.data_schema
            loader_code = CodeGenerator.generate_file_loader_code(filepath, collection, schema.entry_point)
            click.echo(code)
            click.echo('\n\n')
            click.echo(loader_code)

        elif action == 'sample':
            sample = reader.collection(collection).samples()
            click.echo(json.dumps(sample, indent=2))


@click.command()
@click.argument('name')
@click.option('--force', '-f', default=False, help='Force remove dataset without confirmation.', is_flag=True)
@click.pass_obj
def remove(manager: DatasetStorageManager, name, force):
    """remove managed dataset"""
    store_name, dataset_name = parse_dataset_name(name)
    if not force:
        confirmation = click.prompt(f'Are you sure you want to remove dataset {name}? (y/N): ')
        if confirmation.lower() == 'y':
            manager.delete_dataset(store_name, dataset_name)
    else:
        manager.delete_dataset(store_name, dataset_name)


@click.command(name='list')
@click.option('--store', '-s', help='select dataset store space to list.')
@click.pass_obj
def list_datasets(manager: DatasetStorageManager, store):
    """list datasets"""
    items = []
    if store:
        items = manager.list_datasets_in_store(store).items
    else:
        items = manager.list_datasets().items
    click.echo(f'Found {len(items)} datasets:\n')
    for item in items:
        click.echo(f'  {item.datastore}/{item.dataset}')


@click.command(name='import')
@click.argument('name')
@click.option('--url', '-u', help='url of the dataset file to import')
@click.option('--file', '-f', help='local file path of the dataset file to import', type=click.Path())
@click.pass_obj
def import_(manager, name, url, file):
    """import dataset from url or file"""
    if url is not None and file is not None:
        click.echo('Usage: import [NAME] -u [url]')
        click.echo('Usage: import [NAME] -f [file-path]')
        raise click.Abort()

    if url is None and file is None:
        click.echo('Usage: import [NAME] -u [url]')
        click.echo('Usage: import [NAME] -f [file-path]')
        raise click.Abort()

    store_name, dataset_name = parse_dataset_name(name)

    if url is not None:
        click.echo(f'importing remote file from {url}')
        manager.import_url(url, store_name, dataset_name)
    elif file is not None:
        click.echo(f'importing local file from {file}')
        manager.import_file(file, store_name, dataset_name)


# Add subcommands to the main command
cli.add_command(print_info)
cli.add_command(remove)
cli.add_command(list_datasets)
cli.add_command(inspect_file)
cli.add_command(import_)

if __name__ == '__main__':  # pragma: no cover
    cli()
